"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2018 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
}
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Container Generator
 */
const fs_1 = __importDefault(require("fs"));
const fs_extra_1 = require("fs-extra");
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const commander_1 = __importDefault(require("commander"));
const inquirer_1 = __importDefault(require("inquirer"));
const utils_1 = require("../utils");
const package_json_1 = __importDefault(require("../../../package.json"));
const readFile = util_1.promisify(fs_1.default.readFile);
let argName;
commander_1.default
    .version(package_json_1.default.version)
    .arguments('[name]')
    .action((name) => (argName = name))
    .option('-t, --type <type>', 'type of the component (normal|pure)', utils_1.convertTypeArg)
    .option('-h, --headers', 'whether to generate headers')
    .option('-a, --actions', 'whether to generate actions / reducer')
    .option('-s, --saga', 'whether to generate sagas')
    .option('-m, --messages', 'whether to generate i18n messages')
    .parse(process.argv);
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        const actionsTemplate = yield readFile(require.resolve('./actions.ts.hbs'));
        const classTemplate = yield readFile(require.resolve('./class.tsx.hbs'));
        const loaderTemplate = yield readFile(require.resolve('./loader.ts.hbs'));
        const messagesTemplate = yield readFile(require.resolve('./messages.ts.hbs'));
        const reducerTemplate = yield readFile(require.resolve('./reducer.ts.hbs'));
        const sagaTemplate = yield readFile(require.resolve('./saga.ts.hbs'));
        const selectorsTemplate = yield readFile(require.resolve('./selectors.ts.hbs'));
        const questions = [];
        let type = commander_1.default.type;
        if (!type) {
            questions.push({
                type: 'list',
                name: 'type',
                message: 'Select the type of the component',
                default: 'React.PureComponent',
                choices: ['React.PureComponent', 'React.Component'],
            });
        }
        let name = argName;
        if (!name) {
            questions.push({
                name: 'name',
                message: 'What should it be called?',
                validate: (value) => {
                    if (/.+/.test(value)) {
                        return utils_1.componentExists(value)
                            ? 'A component or container with this name already exists'
                            : true;
                    }
                    return 'The name is required';
                },
            });
        }
        let wantHeaders = commander_1.default.headers;
        if (wantHeaders === undefined) {
            questions.push({
                type: 'confirm',
                name: 'wantHeaders',
                default: true,
                message: 'Do you want headers (i.e., react-helmet)?',
            });
        }
        let wantActionsAndReducer = commander_1.default.actions;
        if (wantActionsAndReducer === undefined) {
            questions.push({
                type: 'confirm',
                name: 'wantActionsAndReducer',
                default: true,
                message: 'Do you want an actions/constants/selectors/reducer tuple for this container?',
            });
        }
        let wantSaga = commander_1.default.saga;
        if (wantSaga === undefined) {
            questions.push({
                type: 'confirm',
                name: 'wantSaga',
                default: true,
                message: 'Do you want sagas for asynchronous flows? (e.g. fetching data)',
            });
        }
        let wantMessages = commander_1.default.messages;
        if (wantMessages === undefined) {
            questions.push({
                type: 'confirm',
                name: 'wantMessages',
                default: true,
                message: 'Do you want i18n messages (i.e., will this component use text)?',
            });
        }
        if (questions.length > 0) {
            const answers = yield inquirer_1.default.prompt(questions);
            if (answers.type) {
                type = answers.type;
            }
            if (answers.name) {
                name = answers.name;
            }
            if (answers.wantHeaders) {
                wantHeaders = answers.wantHeaders;
            }
            if (answers.wantActionsAndReducer) {
                wantActionsAndReducer = answers.wantActionsAndReducer;
            }
            if (answers.wantSaga) {
                wantSaga = answers.wantSaga;
            }
            if (answers.wantMessages !== undefined) {
                wantMessages = answers.wantMessages;
            }
        }
        const context = {
            name,
            type,
            wantHeaders,
            wantActionsAndReducer,
            wantSaga,
            wantMessages,
        };
        const componentDir = path_1.default.resolve(process.cwd(), `src/containers/${name}`);
        yield fs_extra_1.mkdirs(path_1.default.resolve(componentDir, 'tests'));
        yield utils_1.renderTemplate(classTemplate, path_1.default.resolve(componentDir, 'index.tsx'), context);
        yield utils_1.renderTemplate(loaderTemplate, path_1.default.resolve(componentDir, 'loader.ts'), context);
        if (wantMessages) {
            yield utils_1.renderTemplate(messagesTemplate, path_1.default.resolve(componentDir, 'messages.ts'), context);
        }
        if (wantActionsAndReducer) {
            yield utils_1.renderTemplate(actionsTemplate, path_1.default.resolve(componentDir, 'actions.ts'), context);
            yield utils_1.renderTemplate(selectorsTemplate, path_1.default.resolve(componentDir, 'selectors.ts'), context);
            yield utils_1.renderTemplate(reducerTemplate, path_1.default.resolve(componentDir, 'reducer.ts'), context);
        }
        if (wantSaga) {
            yield utils_1.renderTemplate(sagaTemplate, path_1.default.resolve(componentDir, 'saga.ts'), context);
        }
    });
}
run()
    .then(() => {
    process.exit(0);
})
    .catch((err) => {
    console.log('Error running script.', err);
    process.exit(1);
});
//# sourceMappingURL=index.js.map